/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.definition;

import org.ow2.orchestra.facade.data.runtime.BpelFaultData;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.services.MessageCarrierRepository;
import org.ow2.orchestra.services.itf.Replier;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author S. Ali Tokmen
 */
public class FaultInCHHandler extends ExceptionHandler {

  private static final long serialVersionUID = 6330814889501735922L;

  private static final Logger LOG = Logger.getLogger(FaultInCHHandler.class.getName());

  @Override
  public boolean matches(final Class< ? > exceptionClass) {
    return BpelFaultException.class.isAssignableFrom(exceptionClass);
  }

  @Override
  protected void executeHandler(final BpelExecution execution, final Exception exception,
    final CompositeElement processElementImpl) {
    final Node chNode = (Node) processElementImpl;
    final BpelExecution bpelExecution = execution;
    final BpelFaultException bpelFault = (BpelFaultException) exception;

    EnvTool.getRecorder().recordExceptionOccured(new BpelFaultData(bpelFault, bpelExecution.getActivityInstanceUUID()));

    BpelExecution chBpelExecution = bpelExecution;
    while (chBpelExecution != null && chBpelExecution.getNode() != chNode) {
      chBpelExecution = chBpelExecution.getParent();
    }
    if (chBpelExecution == null || chBpelExecution.getNode() != chNode) {
      Misc.unreachableStatement("Cannot find the bpelExecution for compensation handler " + chNode);
      return;
    }
    final ScopeRuntime scopeRuntime = chBpelExecution.getCurrentScopeRuntime();

    Misc.fastDynamicLog(FaultInCHHandler.LOG, Level.FINEST,
        "Handling exception %s in BpelExecution %s: scopeRuntime state is %s, CHBpelExecution is %s",
        exception, bpelExecution, scopeRuntime.getState(), chBpelExecution);

    if (!ScopeState.COMPENSATING.equals(scopeRuntime.getState())) {
      Misc.unreachableStatement("The compensation handler " + chNode + " is in invalid state " + scopeRuntime.getState());
    }

    // close pending connections
    for (final Replier replier : scopeRuntime.getRepliers()) {
      MessageCarrierRepository.throwMissingReply(replier.getOpenIMAKey());
    }

    scopeRuntime.setLastException(bpelFault.fullCopy());
    chBpelExecution.terminate();
  }
}
