/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import org.ow2.orchestra.definition.Node;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ExecutionState;

import java.util.Map;

/**
 * @author S. Ali Tokmen
 */
public abstract class ActivityWithSingleChild extends AbstractActivity {

  private static final long serialVersionUID = 5280723485494800792L;

  protected Node child;

  protected ActivityWithSingleChild(final Node child, final ActivityDefinitionUUID uuid, final String name) {
    this.child = child;
    this.setUUID(uuid);
    this.setName(name);
  }

  /**
   *
   */
  protected ActivityWithSingleChild() {
    // for hibernate
  }

  public final void execute(final BpelExecution execution) {
    final BpelExecution bpelExecution = execution;
    this.beforeRunning(bpelExecution);
    this.executeActivity(bpelExecution);
  }

  public void executeActivity(final BpelExecution bpelExecution) {
    bpelExecution.createExecution().execute(this.child);
  }

  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      // this is the child execution returning
      // end child execution
      execution.end(ExecutionState.ended);

      final BpelExecution parentExecution = execution.getParent();

      // remove finished execution from parent
      parentExecution.removeExecution(execution);

      // signal must be non-null to make the distinction with child
      // executions in the first line of this method
      parentExecution.signal("finished");
    } else if (signal.equals("finished")) {
      this.afterRunned(execution);
      this.signal(execution);
    }
  }

  protected abstract void signal(BpelExecution execution);
}
