/*
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 */

package org.ow2.orchestra.definition.activity;

import org.ow2.orchestra.runtime.BpelExecution;

import java.util.Map;

/**
 * @author : manhtoan
 */
public interface InterfaceActivity {
  /**
   * piece of Java code that is to be included in a process as node behaviour or
   * as a hidden listener to process events.
   *
   * <p>
   * Activities can be used to implement the behaviour of nodes, in which case
   * this behaviour is associated to a graphical node in the diagram, or they
   * can be added as events, in that case, they are being hidden from the
   * diagram.
   * </p>
   *
   */
  void execute(BpelExecution execution);

  /**
   * handles an external trigger.
   *
   * <p>
   * An external trigger that comes into an execution through one of the
   * {@link BpelExecution#signal()} methods, will be delegated to the node in which
   * the execution is positioned when it receives the external trigger.
   * </p>
   *
   * <p>
   * The signal method implements how the node will react on that signal. For
   * example, the outgoing transition could be taken that corresponds with the
   * given signal.
   * </p>
   *
   * @param execution
   *          the {@link BpelExecution} for which the signal is given
   *
   * @param signal
   *          is an abstract text that can be associated with a signal. this
   *          corresponds to e.g. a method name in a java class interface. The
   *          implementation can decide e.g. to use the signal to identify the
   *          outgoing transition.
   *
   * @param parameters
   *          is extra information that can be provided with a signal. In this
   *          way, it is somewhat similar to the parameters that can be fed into
   *          a method invocation through reflection.
   *
   */
  void signal(
      final BpelExecution execution,
      final String signal,
      final Map<String, Object> parameters);
}
