/*
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 */
package org.ow2.orchestra.env.binding;

import org.ow2.orchestra.env.descriptor.HibernateEntityManagerFactoryDescriptor;
import org.ow2.orchestra.env.descriptor.PropertiesDescriptor;
import org.ow2.orchestra.util.XmlUtil;
import org.ow2.orchestra.xml.Parse;
import org.ow2.orchestra.xml.Parser;
import org.w3c.dom.Element;

import java.util.List;

/**
 * parses a descriptor for creating a hibernate SessionFactory.
 *
 * See schema docs for more details.
 *
 * @author Tom Baeyens
 */
public class HibernateEntityManagerFactoryBinding extends WireDescriptorBinding {

  private static final PropertiesBinding PROPERTIES_BINDING = new PropertiesBinding();

  public HibernateEntityManagerFactoryBinding() {
    super("hibernate-entity-manager-factory");
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    final HibernateEntityManagerFactoryDescriptor descriptor = new HibernateEntityManagerFactoryDescriptor();

    if (element.hasAttribute("persistence-unit")) {
      final String value = element.getAttribute("persistence-unit");
      descriptor.setPersistenceUnitName(value);
    } else {
      parse.addProblem("attribute 'persistence-unit' is required in element '"
              + XmlUtil.getTagLocalName(element) + "': "
              + XmlUtil.toString(element), element);
    }

    final List<Element> configElements = XmlUtil.elements(element);
    if (configElements != null) {
      for (final Element configElement : configElements) {
        if ("properties".equals(XmlUtil.getTagLocalName(configElement))) {
          final PropertiesDescriptor propertiesDescriptor =
                  (PropertiesDescriptor) HibernateEntityManagerFactoryBinding.PROPERTIES_BINDING.parse(configElement, parse, parser);
          descriptor.setPropertiesDescriptor(propertiesDescriptor);
        } else {
          parse.addProblem("unknown hibernate configuration element " + XmlUtil.toString(configElement), configElement);
        }
      }
    }

    return descriptor;
  }

}
