/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.util.Set;

import org.ow2.orchestra.env.EnvironmentFactory;
import org.ow2.orchestra.facade.RuntimeAPI;
import org.ow2.orchestra.facade.data.def.ActivityDefinitionData;
import org.ow2.orchestra.facade.data.def.ProcessDefinitionData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.jmx.commands.GetActivityDefinitionsCommand;
import org.ow2.orchestra.jmx.commands.GetInstanceCommand;
import org.ow2.orchestra.jmx.commands.GetInstancesCommand;
import org.ow2.orchestra.jmx.commands.GetInstancesOfProcessCommand;
import org.ow2.orchestra.jmx.commands.GetProcessCommand;
import org.ow2.orchestra.jmx.commands.GetProcessesCommand;
import org.ow2.orchestra.services.commands.CommandService;


/**
 * @author Guillaume Porcher
 *
 */
public class RuntimeAPIImpl implements RuntimeAPI {

  private final EnvironmentFactory environmentFactory;

  public RuntimeAPIImpl(final EnvironmentFactory environmentFactory) {
    this.environmentFactory = environmentFactory;
  }

  protected CommandService getCommandService() {
    return this.environmentFactory.get(CommandService.class);
  }

  public Set<ProcessDefinitionData> getProcesses() {
    return this.getCommandService().execute(new GetProcessesCommand());
  }

  public ProcessDefinitionData getProcess(final ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new GetProcessCommand(processUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      } else {
        throw e;
      }
    }
  }

  public Set<ActivityDefinitionData> getActivityDefinitions(final ProcessDefinitionUUID processUUID)
    throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new GetActivityDefinitionsCommand(processUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      } else {
        throw e;
      }
    }
  }

  public Set<ProcessInstanceData> getProcessInstances() {
    return this.getCommandService().execute(new GetInstancesCommand());
  }

  public Set<ProcessInstanceData> getProcessInstances(final ProcessDefinitionUUID processDefinitionUUID)
    throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new GetInstancesOfProcessCommand(processDefinitionUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      } else {
        throw e;
      }
    }
  }

//  /* (non-Javadoc)
//   * @see org.ow2.orchestra.facade.RuntimeAPI
//  #getProcessInstances(org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID, org.ow2.orchestra.facade.runtime.ActivityState)
//   */
//  public Set<ProcessInstanceData> getProcessInstances(
//  final ProcessDefinitionUUID processDefinitionUUID, final ActivityState processInstanceState)
//      throws ProcessNotFoundException {
//    // TODO Auto-generated method stub
//    return null;
//  }
//
//  /* (non-Javadoc)
//   * @see org.ow2.orchestra.facade.RuntimeAPI#getProcessInstances(org.ow2.orchestra.facade.runtime.ActivityState)
//   */
//  public Set<ProcessInstanceData> getProcessInstances(final ActivityState processInstanceState) {
//    // TODO Auto-generated method stub
//    return null;
//  }


  public ProcessInstanceData getProcessInstance(final ProcessInstanceUUID processInstanceUUID) throws InstanceNotFoundException {
    try {
      return this.getCommandService().execute(new GetInstanceCommand(processInstanceUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof InstanceNotFoundException) {
        throw (InstanceNotFoundException) e.getCause();
      } else {
        throw e;
      }
    }
  }

//  /* (non-Javadoc)
//   * @see org.ow2.orchestra.facade.RuntimeAPI#getRunningActivityInstances(org.ow2.orchestra.facade.uuid.ProcessInstanceUUID)
//   */
//  public Set<ActivityInstanceData> getRunningActivityInstances(
//  final ProcessInstanceUUID processUUID) throws InstanceNotFoundException {
//    // TODO Auto-generated method stub
//    return null;
//  }
//
//  /* (non-Javadoc)
//   * @see org.ow2.orchestra.facade.RuntimeAPI#getActivityInstance(org.ow2.orchestra.facade.uuid.ActivityInstanceUUID)
//   */
//  public ActivityInstanceData getActivityInstance(
//  final ActivityInstanceUUID activityInstanceUUID) throws ActivityInstanceNotFoundException {
//    // TODO Auto-generated method stub
//    return null;
//  }

}
