/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import org.ow2.orchestra.env.Environment;
import org.ow2.orchestra.facade.criteria.PagedResult;
import org.ow2.orchestra.facade.criteria.Criteria;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.orchestra.services.QuerierPagedResult;
import org.ow2.orchestra.services.Querier;
import org.ow2.orchestra.services.commands.Command;
import org.ow2.orchestra.util.EnvTool;

/**
 * Command used to retrieve {@link org.ow2.orchestra.facade.criteria.PagedResult} according to
 * {@link Criteria}
 * 
 * @author Loic Albertin
 * 
 */
public final class FindProcessInstancesPageCommand implements Command<PagedResult> {

  private static final long serialVersionUID = -1397425437666885245L;

  private Criteria<ProcessInstance> criteria;

  /**
   * Default constructor
   * 
   * @param criteria {@link Criteria} used to retrieve
   *          {@link org.ow2.orchestra.facade.criteria.PagedResult}
   */
  public FindProcessInstancesPageCommand(Criteria<ProcessInstance> criteria) {
    this.criteria = criteria;
  }

  /**
   * This implementation uses the {@link Querier} returned by
   * {@link EnvTool#getQuerier()} to retrieve a
   * {@link org.ow2.orchestra.services.QuerierPagedResult} object. {@link ProcessFullInstance}s
   * are then extracted and transformed as {@link ProcessInstanceImpl} returned
   * in {@link org.ow2.orchestra.facade.criteria.PagedResult}.
   * 
   * {@inheritDoc}
   */
  public PagedResult<ProcessInstance> execute(final Environment environment) {
    final PagedResult<ProcessInstance> result = new PagedResult<ProcessInstance>();
    final Querier querier = EnvTool.getQuerier();
    // Querier could be null with the QuerierChainer implementation
    if (querier != null) {
      QuerierPagedResult<ProcessFullInstance> querierPage = querier.findProcessInstances(criteria);
      result.setTotalElementsCount(querierPage.getTotalElementsCount());
      for (final ProcessFullInstance processFullInstance : querierPage.getElements()) {
        result.getElements().add(new ProcessInstanceImpl(processFullInstance));
      }
    }
    return result;
  }
}
