/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.ow2.orchestra.env.Environment;
import org.ow2.orchestra.facade.data.runtime.PendingMessageData;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.commands.Command;
import org.ow2.orchestra.util.EnvTool;

/**
 * @author Guillaume Porcher
 *
 */
public final class GetPendingMessagesCommand implements Command<List<PendingMessageData>> {

  private static final long serialVersionUID = 8597363585801154662L;

  public List<PendingMessageData> execute(final Environment environment) {
    final List<PendingMessageData> result = new ArrayList<PendingMessageData>();
    // Get the list of pending messages from the repository
    final Collection<PendingMessage> messages = EnvTool.getRepository().getPendingMessages();
    // For each message, create a pendingMessageData
    if (messages != null) {
      for (final PendingMessage message : messages) {
        // Create message data
        final PendingMessageData messageData = new PendingMessageData(
            message.getDbid(),
            message.getMessage().copy(),
            message.getOperationKey().getProcessQName(),
            message.getOperationKey().getPortTypeQName(),
            message.getOperationKey().getOperationName(),
            message.getMessageCarrierId() == null || message.getMessageCarrierId().equals(""),
            message.getAddedDate()
            );
        // Add to results
        result.add(messageData);
      }
    }
    // return results
    return result;
  }
}
