/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.Node;
import org.ow2.orchestra.definition.activity.AbstractActivity;
import org.ow2.orchestra.definition.activity.ExtensionActivity;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.parsing.BpelParser;
import org.ow2.orchestra.util.XmlUtil;
import org.ow2.orchestra.xml.Parse;
import org.ow2.orchestra.xml.Parser;
import org.w3c.dom.Element;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ExtensionActivityBinding extends BpelActivityBinding {

  /**
   * log.
   */
  private static Logger log = Logger.getLogger(ExtensionActivityBinding.class.getName());


  public ExtensionActivityBinding() {
    super("extensionActivity", ActivityType.EXTENSION_ACTIVITY);
  }

  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element dom element to parse
   * @param parse   parse
   * @param parser  parser
   * @return the build behaviour!
   */
  public Node parse(final Element element, final Parse parse, final Parser parser) {
    if (ExtensionActivityBinding.log.isLoggable(Level.FINE)) {
      ExtensionActivityBinding.log.fine("parsing element = " + element);
    }
    ExtensionActivity extensionActivity = new ExtensionActivity();
    this.parseStandardAttributes(element,extensionActivity,parse);
    ActivityBinding.parseExtensionAttributes(element, null, parse);
    this.parseStandardElements(element,extensionActivity,parse);
    ActivityBinding.parseExtensionElements(element, null, parse);
    final Node extensionActivityNode = this.createActivityNode(parse, parser, extensionActivity);
    final List<Element> childrenElements = XmlUtil.elements(element);
    if (childrenElements.size() > 1) {
      throw new OrchestraRuntimeException("Only one Element is expected as a child of extensionActivity");
    }
    final Element extensionActivityToParse = childrenElements.get(0);
    extensionActivity.setActivity((AbstractActivity) parser.parseElement(extensionActivityToParse, parse, BpelParser.CATEGORY_EXTENSION_ACTIVITY));
    BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
    bpelProcess.getExtensionActivities().add(extensionActivity);

    return extensionActivityNode;
  }


}
