/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db.hibernate;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Properties;

import org.hibernate.HibernateException;
import org.hibernate.usertype.ParameterizedType;
import org.hibernate.usertype.UserType;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class GenericEnumUserType implements UserType, ParameterizedType {

  @SuppressWarnings("unchecked")
  private Class< ? extends Enum> enumClass;

  private static final int[] SQL_TYPES = new int[] { Types.VARCHAR };


  public void setParameterValues(final Properties parameters) {
    final String enumClassName = parameters.getProperty("enumClass");
    try {
      this.enumClass = Class.forName(enumClassName).asSubclass(Enum.class);
    } catch (final ClassNotFoundException cfne) {
      throw new HibernateException("Enum class not found", cfne);
    }
  }

  public Class< ? > returnedClass() {
    return this.enumClass;
  }

  public Object nullSafeGet(final ResultSet rs, final String[] names, final Object owner)
    throws SQLException {
    final String identifier = rs.getString(names[0]);
    if (identifier == null) {
      return null;
    }
    return Enum.valueOf(this.enumClass, identifier);
  }

  public void nullSafeSet(final PreparedStatement st, final Object value, final int index)
    throws SQLException {
    if (value == null) {
      st.setNull(index, GenericEnumUserType.SQL_TYPES[0]);
    } else {
      final String identifier = ((Enum) value).name();
      st.setString(index, identifier);
    }
  }

  public int[] sqlTypes() {
    return GenericEnumUserType.SQL_TYPES;
  }

  public Object assemble(final Serializable cached, final Object owner) {
    return cached;
  }

  public Object deepCopy(final Object value) {
    return value;
  }

  public Serializable disassemble(final Object value) {
    return (Serializable) value;
  }

  public boolean equals(final Object x, final Object y) {
    return x == y;
  }

  public int hashCode(final Object x) {
    return x.hashCode();
  }

  public boolean isMutable() {
    return false;
  }

  public Object replace(final Object original, final Object target, final Object owner) {
    return original;
  }
}
