/*******************************************************************************
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.rest;

import org.ow2.orchestra.env.EnvironmentFactory;
import org.ow2.orchestra.facade.QueryDefinitionAPI;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.exception.ActivityNotFoundException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.rest.ByteArrayWrapper;
import org.ow2.orchestra.facade.rest.CollectionWrapper;
import org.ow2.orchestra.facade.rest.InterfaceWrapper;
import org.ow2.orchestra.facade.rest.JaxRSQueryDefinitionAPI;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.jmx.ClassLoaderInterceptor;
import org.ow2.orchestra.jmx.QueryDefinitionAPIImpl;

import javax.xml.namespace.QName;
import java.lang.reflect.Proxy;

/**
 * @author Guillaume Porcher
 */
public class JaxRSQueryDefinitionAPIImpl implements JaxRSQueryDefinitionAPI {

  private final QueryDefinitionAPI queryDefinitionAPI;

  public JaxRSQueryDefinitionAPIImpl(final EnvironmentFactory environmentFactory) {
    final QueryDefinitionAPI realApi = new QueryDefinitionAPIImpl(environmentFactory);
    final ClassLoaderInterceptor clInterceptor = new ClassLoaderInterceptor(realApi);
    this.queryDefinitionAPI = (QueryDefinitionAPI) Proxy.newProxyInstance(
            JaxRSQueryDefinitionAPIImpl.class.getClassLoader(),
            new Class[]{QueryDefinitionAPI.class},
            clInterceptor);
  }

  // QUERIER DEFINITION API

  public CollectionWrapper<ProcessDefinition> findProcessDefinitions(
          final String processNamespace,
          final String processName,
          final ProcessState state) {
    if (processName != null && processNamespace != null) {
      QName processQName = new QName(processNamespace, processName);
      if (state != null) {
        return CollectionWrapper.make(this.queryDefinitionAPI.findProcessDefinitions(processQName, state));
      }
      return CollectionWrapper.make(this.queryDefinitionAPI.findProcessDefinitions(processQName));
    } else if (state != null) {
      return CollectionWrapper.make(this.queryDefinitionAPI.findProcessDefinitions(state));
    }
    return CollectionWrapper.make(this.queryDefinitionAPI.findProcessDefinitions());
  }

  public InterfaceWrapper<ProcessDefinition> getProcessDefinition(final ProcessDefinitionUUID processUUID) throws
          ProcessNotFoundException {
    return InterfaceWrapper.make(this.queryDefinitionAPI.getProcessDefinition(processUUID));
  }

  public InterfaceWrapper<ActivityDefinition> getActivityDefinition(final ActivityDefinitionUUID activityDefUUID) throws
          ActivityNotFoundException {
    return InterfaceWrapper.make(this.queryDefinitionAPI.getActivityDefinition(activityDefUUID));
  }

  public CollectionWrapper<ActivityDefinition> findActivityDefinitions(final ProcessDefinitionUUID processUUID)
          throws ProcessNotFoundException {
    return CollectionWrapper.make(this.queryDefinitionAPI.findActivityDefinitions(processUUID));
  }

  public ByteArrayWrapper getProcessResource(final ProcessDefinitionUUID processUUID, final String resourceName)
          throws ProcessNotFoundException {
    return ByteArrayWrapper.make(this.queryDefinitionAPI.getProcessResource(processUUID, resourceName));
  }


}
