/*******************************************************************************
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.rest;

import org.ow2.orchestra.env.EnvironmentFactory;
import org.ow2.orchestra.facade.QueryRuntimeAPI;
import org.ow2.orchestra.facade.criteria.Criteria;
import org.ow2.orchestra.facade.criteria.PagedResult;
import org.ow2.orchestra.facade.exception.ActivityInstanceNotFoundException;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.rest.CollectionWrapper;
import org.ow2.orchestra.facade.rest.InterfaceWrapper;
import org.ow2.orchestra.facade.rest.JaxRSQueryRuntimeAPI;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.jmx.ClassLoaderInterceptor;
import org.ow2.orchestra.jmx.QueryRuntimeAPIImpl;

import java.lang.reflect.Proxy;

/**
 * @author Guillaume Porcher
 */
public class JaxRSQueryRuntimeAPIImpl implements JaxRSQueryRuntimeAPI {

  private final QueryRuntimeAPI queryRuntimeAPI;

  public JaxRSQueryRuntimeAPIImpl(final EnvironmentFactory environmentFactory) {
    final QueryRuntimeAPI realApi = new QueryRuntimeAPIImpl(environmentFactory);
    final ClassLoaderInterceptor clInterceptor = new ClassLoaderInterceptor(realApi);
    this.queryRuntimeAPI = (QueryRuntimeAPI) Proxy.newProxyInstance(
            JaxRSQueryRuntimeAPIImpl.class.getClassLoader(),
            new Class[]{QueryRuntimeAPI.class},
            clInterceptor);
  }

  public CollectionWrapper<ActivityInstance> findActivityInstances(
          ProcessInstanceUUID instanceUUID,
          ActivityState activityState)
          throws InstanceNotFoundException {
    if (activityState != null) {
      return CollectionWrapper.make(this.queryRuntimeAPI.findActivityInstances(instanceUUID, activityState));
    }
    return CollectionWrapper.make(this.queryRuntimeAPI.findActivityInstances(instanceUUID));
  }

  public InterfaceWrapper<ActivityInstance> getActivityInstance(ActivityInstanceUUID activityInstanceUUID) throws
          ActivityInstanceNotFoundException {
    return InterfaceWrapper.make(this.queryRuntimeAPI.getActivityInstance(activityInstanceUUID));
  }

  public CollectionWrapper<ProcessInstance> findProcessInstances(
          ProcessDefinitionUUID processDefinitionUUID,
          ActivityState processInstanceState) throws ProcessNotFoundException {
    if (processDefinitionUUID != null) {
      if (processInstanceState != null) {
        return CollectionWrapper.make(this.queryRuntimeAPI.findProcessInstances(processDefinitionUUID, processInstanceState));
      }
      return CollectionWrapper.make(this.queryRuntimeAPI.findProcessInstances(processDefinitionUUID));
    } else if (processInstanceState != null) {
      return CollectionWrapper.make(this.queryRuntimeAPI.findProcessInstances(processInstanceState));
    }
    return CollectionWrapper.make(this.queryRuntimeAPI.findProcessInstances());
  }

  public InterfaceWrapper<ProcessInstance> getProcessInstance(ProcessInstanceUUID processInstanceUUID)
          throws InstanceNotFoundException {
    return InterfaceWrapper.make(this.queryRuntimeAPI.getProcessInstance(processInstanceUUID));
  }

  public PagedResult<ProcessInstance> findProcessInstancesPage(Criteria<ProcessInstance> criteria) {
    return this.queryRuntimeAPI.findProcessInstancesPage(criteria);
  }

}
