/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;

import org.ow2.orchestra.facade.data.runtime.TransitionConditionUpdateData;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.util.EnvTool;


/**
 * @author Charles Souillard
 */
public class LinkRuntime {

  public enum LinkStatus {
    TRUE(Boolean.TRUE), FALSE(Boolean.FALSE), UNSET(null);

    private Boolean booleanValue;
    private LinkStatus(final Boolean booleanValue) {
      this.booleanValue = booleanValue;
    }
    /**
     * @return the booleanValue
     */
    public Boolean getBooleanValue() {
      return this.booleanValue;
    }
    /**
     * @param valueOf
     * @return
     */
    public static LinkStatus get(final Boolean valueOf) {
      if (Boolean.TRUE == valueOf) {
        return TRUE;
      }
      if (Boolean.FALSE == valueOf) {
        return FALSE;
      }
      return UNSET;
    }
  }

  protected long dbid;
  protected int dbversion;

  /** status of this link. */
  protected LinkStatus status = LinkStatus.UNSET;

  protected String name;

  /**
   * execution that waits for this link. Null if no one is waiting for this
   * link.
   */
  protected BpelExecution waitingExecution = null;

  protected ActivityInstanceUUID flowActivityInstanceUUID;

  protected LinkRuntime() {
  }

  public LinkRuntime(final String name, final ActivityInstanceUUID flowActivityInstanceUUID) {
    this.name = name;
    this.flowActivityInstanceUUID = flowActivityInstanceUUID;
  }

  public LinkStatus getStatus() {
    return this.status;
  }

  public void setStatus(final LinkStatus status) {
    this.status = status;
    EnvTool.getRecorder().recordTransitionConditionUpdated(
      new TransitionConditionUpdateData(this.flowActivityInstanceUUID, status.getBooleanValue(), this.name));
  }

  public BpelExecution getWaitingExecution() {
    return this.waitingExecution;
  }

  public void setWaitingExecution(final BpelExecution waitingExecution) {
    this.waitingExecution = waitingExecution;
  }
}
