/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.orchestra.runtime.op;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.Node;
import org.ow2.orchestra.definition.activity.AbstractActivity;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.Misc;

/**
 * @author Tom Baeyens
 */
public class Signal implements AtomicOperation {

  private static final Logger LOG = Logger.getLogger(Signal.class.getName());

  private final String signalName;
  private final Map<String, Object> parameters;
  private final Node node;

  public Signal(final String signalName, final Map<String, Object> parameters, final Node node) {
    this.signalName = signalName;
    this.parameters = parameters;
    this.node = node;
  }


  public void perform(final BpelExecution execution) {
    Misc.fastDynamicLog(Signal.LOG, Level.FINEST, "%s signals %s, signal=%s", execution.toString(), this.node, this.signalName);

    final AbstractActivity activity = this.node.getBehaviour();

    try {
      execution.setWaitForSignal(false);

      activity.signal(execution, this.signalName, this.parameters);

    } catch (final Exception e) {
      execution.handleException(this.node, e, execution.toString()
          + " couldn't signal node " + this.node);
    }

    // if exception has been catched and exception handler has not disabled propagation,
    // then proceed.
    if (!execution.isWaitForSignal()) {
      execution.proceed();
    }
  }

  @Override
  public String toString() {
    return "signal(node)";
  }

}
