/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.orchestra.util.stream;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.MalformedURLException;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;

/**
 * @author Tom Baeyens
 */
public class FileStreamSource extends StreamSource {

  protected File file;

  /**
   * @throws PvmException
   *           if file is null
   */
  public FileStreamSource(final File file) {
    if (file == null) {
      throw new OrchestraRuntimeException("file is null");
    }

    try {
      this.name = file.toURI().toURL().toString();
    } catch (final MalformedURLException e) {
      this.name = file.toString();
    }

    this.file = file;
  }

  @Override
  public InputStream openStream() {
    InputStream stream = null;

    try {
      if (!this.file.exists()) {
        throw new OrchestraRuntimeException("file " + this.file + " doesn't exist");
      }
      if (this.file.isDirectory()) {
        throw new OrchestraRuntimeException("file " + this.file + " is a directory");
      }
      stream = new FileInputStream(this.file);

    } catch (final Exception e) {
      throw new OrchestraRuntimeException("couldn't access file " + this.file + ": "
          + e.getMessage(), e);
    }

    return stream;
  }

}
