/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.xml;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import com.sun.org.apache.xerces.internal.xni.Augmentations;
import com.sun.org.apache.xerces.internal.xni.NamespaceContext;
import com.sun.org.apache.xerces.internal.xni.QName;
import com.sun.org.apache.xerces.internal.xni.XMLAttributes;
import com.sun.org.apache.xerces.internal.xni.XMLLocator;
import com.sun.org.apache.xerces.internal.xni.XNIException;

import org.w3c.dom.Node;

/**
 * This class extends the default {@link DOMParser} to enrich each parsed
 * {@link Node} with its line and column numbers within the XML document. Both
 * of line and column numbers are attached to the {@link Node} using the
 * {@link Node#setUserData(String, Object, org.w3c.dom.UserDataHandler)}
 * operation.
 *
 * <li>line number is registered with the "lineNumber" key</li> <li>column
 * number is registered with the "columnNumber" key</li>
 *
 * @author Loic Albertin
 *
 */
public class ExtendedDOMParser extends DOMParser {

  /**
   * The locator used to retrieve line and column numbers of {@link Node}s
   */
  private XMLLocator xmlLocator;

  /*
   * (non-Javadoc)
   *
   * @see
   * com.sun.org.apache.xerces.internal.parsers.AbstractDOMParser#startElement
   * (com.sun.org.apache.xerces.internal.xni.QName,
   * com.sun.org.apache.xerces.internal.xni.XMLAttributes,
   * com.sun.org.apache.xerces.internal.xni.Augmentations)
   */
  @Override
  public void startElement(final QName element, final XMLAttributes attributes, final Augmentations augs) throws XNIException {
    super.startElement(element, attributes, augs);
    Node node = null;
    try {
      node = (Node) this.getProperty("http://apache.org/xml/properties/dom/current-element-node");
    } catch (final org.xml.sax.SAXException ex) {
      // Nothing to do
    }
    if (node != null && this.xmlLocator != null) {
      node.setUserData("lineNumber", Integer.valueOf(this.xmlLocator.getLineNumber()), null);
      node.setUserData("columnNumber", Integer.valueOf(this.xmlLocator.getColumnNumber()), null);
    }
  }

  /*
   * (non-Javadoc)
   *
   * @see
   * com.sun.org.apache.xerces.internal.parsers.AbstractDOMParser#startDocument
   * (com.sun.org.apache.xerces.internal.xni.XMLLocator, java.lang.String,
   * com.sun.org.apache.xerces.internal.xni.NamespaceContext,
   * com.sun.org.apache.xerces.internal.xni.Augmentations)
   */
  @Override
  public void startDocument(final XMLLocator locator, final String encoding,
      final NamespaceContext namespaceContext, final Augmentations augs) throws XNIException {
    super.startDocument(locator, encoding, namespaceContext, augs);
    this.xmlLocator = locator;
    Node node = null;
    try {
      node = (Node) this.getProperty("http://apache.org/xml/properties/dom/current-element-node");
    } catch (final org.xml.sax.SAXException ex) {
      // Nothing to do
    }

    if (node != null && this.xmlLocator != null) {
      node.setUserData("lineNumber", Integer.valueOf(this.xmlLocator.getLineNumber()), null);
      node.setUserData("columnNumber", Integer.valueOf(this.xmlLocator.getColumnNumber()), null);
    }
  }

}
